package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.Assertions.*
import solutions.bitbadger.documents.query.{CountQuery, DeleteQuery, FindQuery}
import solutions.bitbadger.documents.scala.Results
import solutions.bitbadger.documents.scala.extensions.*
import solutions.bitbadger.documents.scala.tests.TEST_TABLE
import solutions.bitbadger.documents.{Configuration, Field, Parameter, ParameterType}

object CustomFunctions:

  def listEmpty(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    db.conn.deleteByFields(TEST_TABLE, Field.exists(Configuration.idField) :: Nil)
    val result = db.conn.customList[JsonDocument](FindQuery.all(TEST_TABLE), Results.fromData)
    assertEquals(0, result.size, "There should have been no results")

  def listAll(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    val result = db.conn.customList[JsonDocument](FindQuery.all(TEST_TABLE), Results.fromData)
    assertEquals(5, result.size, "There should have been 5 results")

  def singleNone(db: ThrowawayDatabase): Unit =
    assertTrue(db.conn.customSingle[JsonDocument](FindQuery.all(TEST_TABLE), Results.fromData).isEmpty,
      "There should not have been a document returned")

  def singleOne(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertTrue(db.conn.customSingle[JsonDocument](FindQuery.all(TEST_TABLE), Results.fromData).isDefined,
      "There should have been a document returned")

  def nonQueryChanges(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.customScalar[Long](CountQuery.all(TEST_TABLE), Results.toCount),
      "There should have been 5 documents in the table")
    db.conn.customNonQuery(s"DELETE FROM $TEST_TABLE")
    assertEquals(0L, db.conn.customScalar[Long](CountQuery.all(TEST_TABLE), Results.toCount),
      "There should have been no documents in the table")

  def nonQueryNoChanges(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(5L, db.conn.customScalar[Long](CountQuery.all(TEST_TABLE), Results.toCount),
      "There should have been 5 documents in the table")
    db.conn.customNonQuery(DeleteQuery.byId(TEST_TABLE, "eighty-two"),
      Parameter(":id", ParameterType.STRING, "eighty-two") :: Nil)
    assertEquals(5L, db.conn.customScalar[Long](CountQuery.all(TEST_TABLE), Results.toCount),
      "There should still have been 5 documents in the table")

  def scalar(db: ThrowawayDatabase): Unit =
    JsonDocument.load(db)
    assertEquals(3L, db.conn.customScalar[Long](s"SELECT 3 AS it FROM $TEST_TABLE LIMIT 1", Results.toCount),
      "The number 3 should have been returned")
